﻿using UnityEngine;
using System.Collections;

public class GBOBeatPanelController : GBasicObjectToInteract 
{
    [SerializeField]
    private AudioClip beat_audioclip;

    [SerializeField]
    [Range(0.25f, 2.0f)]
    private float beat_interval = 0.5f;

    [SerializeField]
    [Range(0.05f, 0.50f)]
    private float error_range = 0.25f;

    private AudioSource beat_audiosource;
    private bool desestabilized = false;
    private float beat_elapsed_time = 0.0f;
    private IEnumerator beat_coroutine;
    private IEnumerator beat_elapsed_time_coroutine;

    public RadiusWave radius_wave;

    public override void Start()
    {
        if (GetComponent<AudioSource>() != null)
        {
            beat_audiosource = GetComponent<AudioSource>();
            if (beat_audioclip != null)
            {
                beat_audiosource.clip = beat_audioclip;
            }
            else Debug.LogError("Hay que tirar el audioclip en GBOBeatPanelController");
        } else Debug.LogError("GBOBeatPanelController no tiene un AudioSource");
    }

    private void Update()
    {
        if (Input.GetKeyDown(KeyCode.O))
        {
            StartBeating();
        }

        if (Input.GetKeyDown(KeyCode.P))
        {
            StopBeating();
        }
    }

    public void StartBeating()
    {
        desestabilized = true;

        // Empieza a contar el tiempo que lleva latiendo
        StopCoroutineIfNotNull(beat_elapsed_time_coroutine);
        beat_elapsed_time_coroutine = StartCountingElapsedTime();
        StartCoroutine(beat_elapsed_time_coroutine);
              
        // Empieza la cadena de beats  
        StartSingleBeat();
    }

    public void StopBeating()
    {
        desestabilized = false;
        StopCoroutineIfNotNull(beat_coroutine);
        StopCoroutineIfNotNull(beat_elapsed_time_coroutine);
    }

    private void StartSingleBeat()
    {
        StopCoroutineIfNotNull(beat_coroutine);
        StartCoroutine(DoBeat());
    }

    public override void InteractLeftClick()
    {
        base.InteractLeftClick();

        float remainder = (beat_elapsed_time % beat_interval);
        float error_value = (beat_interval/2) - Mathf.Abs((beat_interval / 2) - remainder);

        if (error_value <= error_range)
        {
            Debug.Log("ACIERTO - " + error_value);
        }
        else
        {
            Debug.Log("FALLO - " + error_value);
        }
    }

    public void StopCoroutineIfNotNull(IEnumerator _coroutine)
    {
        if (_coroutine != null)
        {
            StopCoroutine(_coroutine);
        }
    }

    IEnumerator StartCountingElapsedTime()
    {
        beat_elapsed_time = 0.0f;
        while (true)
        {
            beat_elapsed_time += Time.deltaTime;            
            if (!desestabilized)
            {                
                break;
            }
            yield return null;
        }
    }

    IEnumerator DoBeat()
    {                
        float current_time = 0.0f;        
        radius_wave.StartBeat(beat_interval);
        beat_audiosource.Play();

        while (true)
        {
            current_time += Time.deltaTime;

            if (current_time >= beat_interval)
            {
                if (desestabilized)
                {
                    StartSingleBeat();
                }
                break;
            }
            yield return null;
        }
    }
}