﻿using UnityEngine;
using System.Collections;
using UnityEngine.Events;
using UnityEngine.SceneManagement;

public class GGameLogic : MonoBehaviour
{

    bool are_lights_on;
    public bool AreLightsOn { get { return are_lights_on; } }

    bool first_time_lights = true;

    int patients_saved;
    int patients_lost;
    int max_patient_lost_permitted = 5;

    int max_mixes_allowed = 9;
    int mixes_used;

    [SerializeField]
    public static float SECONDS_TO_WAIT_AFTER_CORRECT = 2.75f;

    UnityEvent lights_on_first_time = new UnityEvent();
    public UnityEvent OnLightsTurnOnFirstTime
    {
        get { return lights_on_first_time; }
    }

    [SerializeField]
    GPatientReportInteract patient_report;

    GSustanceMixer sustance_mixer;

    [SerializeField]
    GWaveController wave_display;
    GDisplayWave display_logic;

    int waves_correct = 0;

    [SerializeField]
    Animator animator_patient;

    [SerializeField]
    GSphereCorrects spheres_correct;

    [SerializeField]
    GPatientFailsDisplay patients_failed_display;

    [SerializeField]
    float max_time_to_complete_patient = 80f;
    float actual_time;

    [SerializeField]
    GUITimer timer_ui;

    bool has_to_count_down;

    [SerializeField]
    GUICurrentWave current_wave_display;

    [SerializeField]
    GameObject UI_gameover;

    [SerializeField]
    GameObject no_one_img;

    [SerializeField]
    private AudioClip sound_patient_in;
    [SerializeField]
    private AudioClip sound_patient_out;
    [SerializeField]
    private AudioClip sound_scream;
    [SerializeField]
    private AudioClip sound_hydraulic;

    [SerializeField]
    GBOBeatPanelController beat_panel_controller;

    public enum TYPE_NEW_PATIENT
    {
        NORMAL,
        DIE,
        INITIAL,
        MAX_INJECTIONS,
    }

    [SerializeField]
    Light light_patient;
    Color initial_color_patient;

    bool is_game_over;

    [SerializeField]
    GUIHealed ui_healed;

    [SerializeField]
    GUIMaxInjections ui_injections;

    void Awake()
    {
        GSingleton.Instance.GameLogic = this;
    }

    // Use this for initialization
    void Start()
    {
        sustance_mixer = GSingleton.Instance.SustanceMixer;
        sustance_mixer.RefreshUI();
        display_logic = wave_display.GetComponentInParent<GDisplayWave>();
        animator_patient.SetTrigger("InitialPos");
        GSingleton.Instance.WorldInteraction.CanInteract = false;
        GSingleton.Instance.FadeManager.fade_logic.GetComponentInChildren<CanvasGroup>().alpha = 1;
        Invoke("DoFadeIn", 1.5f);
        initial_color_patient = light_patient.color;
    }

    void DoFadeIn()
    {
        GSingleton.Instance.FadeManager.fade_logic.Fade(0f, 2f, FadeInFinished);
    }

    // Update is called once per frame
    void Update()
    {

        if (has_to_count_down)
        {
            actual_time -= Time.deltaTime;
            if (actual_time <= 0f)
            {
                KillPatient();
            }
            timer_ui.SetTime(actual_time);
        }

        if (Input.GetKeyDown(KeyCode.Space) && is_game_over)
        {
            SceneManager.LoadScene(1);
        }
    }

    public void DebugInjectCorrect()
    {
        InjectionCorrect();
    }


    void FadeInFinished()
    {
        GSingleton.Instance.WorldInteraction.CanInteract = true;
    }

    public void NewPatient(TYPE_NEW_PATIENT _type_new_patient)
    {
        Invoke("DoNewPatientAfterDelay", 2.75f);
        sustance_mixer.ResetMixer();
        has_to_count_down = false;
        AudioSource.PlayClipAtPoint(sound_patient_out, Camera.main.transform.position);
        beat_panel_controller.StopBeating();
        if (_type_new_patient == TYPE_NEW_PATIENT.DIE)
        {
            ShowPatientKilled();
            animator_patient.SetTrigger("Die");
        }
        else if (_type_new_patient == TYPE_NEW_PATIENT.NORMAL)
        {
            Invoke("ShowNextPatitnetAfterCorrect", 2.25f);
        }
        else if (_type_new_patient == TYPE_NEW_PATIENT.INITIAL) //INITIAL
        {
            //animator_patient.SetTrigger("InitialPos");
            animator_patient.SetTrigger("In");
            ShowPatientKilled();
            no_one_img.SetActive(false);
        }
        else if (_type_new_patient == TYPE_NEW_PATIENT.MAX_INJECTIONS)
        {
            ShowPatientMaxInjections();
            Invoke("LaunchAnimDieAfterMaxMix", 2.5f);
        }
    }

    void LaunchAnimDieAfterMaxMix()
    {
        animator_patient.SetTrigger("Die");
    }

    void ShowPatientMaxInjections()
    {
        patient_report.KillPatient();
        wave_display.SetWaveParameters(10, 1);
        display_logic.MaxInjections();
    }

    void ShowNextPatitnetAfterCorrect()
    {
        animator_patient.SetTrigger("Out");
        ShowNewPatientWave();
    }

    void DoNewPatientAfterDelay()
    {
        has_to_count_down = true;       
        spheres_correct.ResetSpheres();
        waves_correct = 0;
        mixes_used = 0;
        SetDificultyBasedOnPatients();       
        ui_injections.SetNUmberMaxInjections(max_mixes_allowed - mixes_used);
        current_wave_display.SetNumberInDisplay(waves_correct + 1);
        AudioSource.PlayClipAtPoint(sound_patient_in, Camera.main.transform.position);
        beat_panel_controller.StartBeating();
    }

    void SetDificultyBasedOnPatients()
    {
        if (patients_saved != 0)
        {
            if (patients_saved % 2 == 0)
            {
                max_time_to_complete_patient -= 10f;            
            }
            if (patients_saved % 2 == 0)
            {
                max_mixes_allowed --;
            }
        }

        max_mixes_allowed = Mathf.Clamp(max_mixes_allowed, 4, 10);
        max_time_to_complete_patient = Mathf.Clamp(max_time_to_complete_patient, 30,100);

        actual_time = max_time_to_complete_patient;
    }

    public void ShowPatientKilled()
    {
        patient_report.KillPatient();
        wave_display.SetWaveParameters(10, 1);
        display_logic.ShowPatientKilled();
    }

    public void ShowNewPatientWave()
    {
        patient_report.SetNewPatient();
        wave_display.SetWaveParameters(10, 1);
        display_logic.ShowNextWave();
    }

    void KillPatient()
    {
        patients_failed_display.SetSphereCorrect(patients_lost);
        patients_lost++;
        Debug.Log("Patient Killed");
        if (patients_lost >= max_patient_lost_permitted)
        {
            GameOver();
        }
        else
        {
            NewPatient(TYPE_NEW_PATIENT.DIE);
        }

        StartCoroutine(AnimLightRed());
    }

    IEnumerator AnimLightRed()
    {
        float actual_time = 0f;
        float time_to_new_color = 1f;
        float delta = 1f / time_to_new_color;

        while (true)
        {
            actual_time += delta * Time.deltaTime;
            light_patient.color = Color.Lerp(light_patient.color, Color.red, actual_time);
            if (actual_time >= time_to_new_color)
            {
                break;
            }
            yield return null;
        }

        actual_time = 0f;
        while (true)
        {
            actual_time += delta * Time.deltaTime;
            light_patient.color = Color.Lerp(light_patient.color, initial_color_patient, actual_time);
            if (actual_time >= time_to_new_color)
            {
                break;
            }
            yield return null;
        }
    }

    public void Inject()
    {
        
        if (patient_report != null)
        {
            //Waves are equal
            if (patient_report.Wave.Amplitude == sustance_mixer.ActualWave.Amplitude && patient_report.Wave.Longitude == sustance_mixer.ActualWave.Longitude)
            {
                Debug.Log("Waves are equal");
                InjectionCorrect();
            }
            else
            {
                Debug.Log("Waves are diferents");
                InjectionFailed();
            }

            AudioSource.PlayClipAtPoint(sound_hydraulic, Camera.main.transform.position);

            Invoke("ScreamDelayed", 0.75f);
        }
    }

    public void InjectionCorrect()
    {
        spheres_correct.SetSphereCorrect(waves_correct);
        waves_correct++;
        mixes_used++;
        ui_injections.SetNUmberMaxInjections(max_mixes_allowed - mixes_used);
        Debug.Log("Waves correct " + waves_correct);
        if (waves_correct >= 3)
        {
            patients_saved++;
            NewPatient(TYPE_NEW_PATIENT.NORMAL);
            ui_healed.SetHealed(patients_saved);
        }
        else
        {
            current_wave_display.SetNumberInDisplay(waves_correct + 1);
            ShowNewPatientWave();
        }

        if (mixes_used >= max_mixes_allowed)
        {
            patients_failed_display.SetSphereCorrect(patients_lost);
            patients_lost++;
            Debug.Log("Patient Killed");
            if (patients_lost >= max_patient_lost_permitted)
            {
                GameOver();
            }
            else
            {
                NewPatient(TYPE_NEW_PATIENT.MAX_INJECTIONS);
            }

            StartCoroutine(AnimLightRed());
        }
    }

    private void ScreamDelayed()
    {
        AudioSource.PlayClipAtPoint(sound_scream, Camera.main.transform.position);
    }

    public void InjectionFailed()
    {
        //patients_lost++;
        mixes_used++;
        ui_injections.SetNUmberMaxInjections(max_mixes_allowed - mixes_used);
        if (mixes_used >= max_mixes_allowed)
        {
            patients_failed_display.SetSphereCorrect(patients_lost);
            patients_lost++;
            Debug.Log("Patient Killed");
            if (patients_lost >= max_patient_lost_permitted)
            {
                GameOver();
            }
            else
            {
                NewPatient(TYPE_NEW_PATIENT.MAX_INJECTIONS);
            }

            StartCoroutine(AnimLightRed());
        }
    }

    public void TurnLights(bool _value)
    {
        if (first_time_lights == true && _value == true)
        {
            if (lights_on_first_time != null)
            {
                lights_on_first_time.Invoke();
            }
            NewPatient(TYPE_NEW_PATIENT.INITIAL);
            first_time_lights = false;
        }
        are_lights_on = _value;
    }

    void GameOver()
    {
        animator_patient.SetTrigger("Die");
        GSingleton.Instance.WorldInteraction.CanInteract = false;
        has_to_count_down = false;
        actual_time = max_time_to_complete_patient;
        Invoke("StartFadeOutGameOver", 1f);
        Debug.Log("Game Over");
    }

    void StartFadeOutGameOver()
    {
        GSingleton.Instance.FadeManager.fade_logic.SimpleFadeOut(2f, FadeGameOverFinished);
    }

    void FadeGameOverFinished()
    {
        UI_gameover.SetActive(true);
        UI_gameover.GetComponent<GUIFade>().SimpleFadeOut(2f);
        is_game_over = true;
    }
}
