﻿using UnityEngine;
using System.Collections;

public class GWaveController : MonoBehaviour 
{
    [Range(0.01f, 1.0f)]
    [SerializeField]
    private float scale = 0.5f;

    [Range(10, 50)]
    [SerializeField]
    private int longitude = 15;
    public int Longitude { get { return longitude; } }

    [Range(1, 5)]
    [SerializeField]
    private int amplitude = 1;
    public int Amplitude { get { return amplitude; } }

    [Range(0.0f, 50.0f)]
    [SerializeField]
    private float wave_speed = 25f;

    [Range(0.01f, 1.0f)]
    [SerializeField]
    private float wave_length = 0.9f;

    private int num_vertex = 90;
    private float division_length = 0.01f;
    private LineRenderer wave;

    private void Start()
    {
        // Tengo lo que mide la wave y el espacio que quiero que haya entre vertices. Calculo el número de vertices que necesito
        wave_length = transform.parent.localScale.x;
        num_vertex = (int)(wave_length / 0.01f);
        //Debug.Log("Numero de vertices: " + num_vertex);

        if (this.GetComponent<LineRenderer>() != null)
        {
            wave = this.GetComponent<LineRenderer>();
            wave.SetVertexCount(num_vertex);
        }

        //division_length = wave_length / (float)num_vertex;
    }

    private void Update()
    {   
        if (Input.GetKeyDown(KeyCode.Q))
        {
            AddAmplitude(-1);
        }
        if (Input.GetKeyDown(KeyCode.W))
        {
            AddAmplitude(1);
        }

        if (Input.GetKeyDown(KeyCode.A))
        {
            AddLongitude(-1);
        }
        if (Input.GetKeyDown(KeyCode.S))
        {
            AddLongitude(1);
        }

        UpdateWave();
    }

    private void UpdateWave()
    {
        // Se actualiza la onda con los valores actuales de longitud y amplitud
        for (int i = 0; i < num_vertex; i++)
        {
            /*wave.SetPosition(i, new Vector3((this.transform.position.x + i * division_length),
                                            (this.transform.position.y + Mathf.Sin((i + Time.realtimeSinceStartup * wave_speed) * Longitude2Frequency(longitude) / scale) * (float)amplitude) * scale,
                                            (this.transform.position.z)));*/

            Vector3 world_coords = new Vector3(i * division_length,
                                              (Mathf.Sin((i + Time.realtimeSinceStartup * wave_speed) * Longitude2Frequency(longitude) / scale) * (float)amplitude) * scale,
                                              0.0f);

            Vector3 x = transform.right * world_coords.x;
            Vector3 y = transform.up * world_coords.y;
            Vector3 z = transform.forward * world_coords.z;

            Vector3 local_coords = transform.position + x + y + z;

            wave.SetPosition(i, local_coords);
        }
    }

    private float Longitude2Frequency(int _longitude)
    {
        return Mathf.PI * 0.1f / (float)_longitude;
    }    

    public void SetWaveParameters(int _longitude, int _amplitude)
    {
        longitude = _longitude;
        amplitude = _amplitude;
    }

    public void AddAmplitude(int _amp_modules)
    {
        int amp_after_apply = amplitude + _amp_modules;
        if (amp_after_apply >= 1 && amp_after_apply <= 5)
        {
            amplitude += _amp_modules;
        }
    }

    public void AddLongitude(int _long_modules)
    {
        int long_after_apply = longitude + _long_modules * 10;
        if (long_after_apply >= 10 && long_after_apply <= 50)
        {
            longitude = long_after_apply;
        }
    }
}