﻿using UnityEngine;
using System.Collections;

public class GObjectCloseCamera : GBasicObjectToInteract {

    [SerializeField]
    float distance_camera = 0.75f;

    [SerializeField]
    float speed_move_to_pos = 1f;

    [SerializeField]
    float speed_rotate_to_pos = 1f;

    protected bool is_in_front_of_camera;

    Vector3 initial_pos;
    Quaternion initial_rot;

	// Use this for initialization
	public override void Start () {

        initial_pos = transform.position;
        initial_rot = transform.rotation;
        GSingleton.Instance.WorldInteraction.OnEscape.AddListener(Escape);
	}
	
	// Update is called once per frame
	public virtual void Update () {

        if (is_in_front_of_camera == true)
        {
            PlaceInFrontOfcamera();
        }
	}

    public override void InteractLeftClick()
    {
        base.InteractLeftClick();
        if (is_in_front_of_camera == false)
        {
            ForcePlaceInFront();
            is_in_front_of_camera = true;
            ReleaseOver();
        }
    }

    public override void InteractRightClick()
    {
        base.InteractRightClick();
        if (is_in_front_of_camera == true)
        {
            ReturnToPlace();
            is_in_front_of_camera = false;
        }
    }

    public override void Escape()
    {
        base.Escape();
        if (is_in_front_of_camera == true)
        {
            ReturnToPlace();
            is_in_front_of_camera = false;
        }
    }

    void ForcePlaceInFront()
    {
        transform.position = Camera.main.transform.position + Camera.main.transform.forward * distance_camera;
        transform.rotation = Quaternion.LookRotation(Camera.main.transform.position - transform.position, Vector3.up);
    }

    void PlaceInFrontOfcamera()
    {
        transform.position = Vector3.MoveTowards(transform.position,Camera.main.transform.position + Camera.main.transform.forward * distance_camera,speed_move_to_pos * Time.deltaTime);
        transform.rotation = Quaternion.RotateTowards(transform.rotation,Quaternion.LookRotation(Camera.main.transform.position - transform.position,Vector3.up),speed_rotate_to_pos * Time.deltaTime);

    }

    void ReturnToPlace()
    {
        transform.position = initial_pos;
        transform.rotation = initial_rot;
    }

    public override void OnOver()
    {
        if(is_in_front_of_camera == false)
            base.OnOver();
    }

}
