﻿using UnityEngine;
using System.Collections;
using System.Collections.Generic;

public class GSustanceMixer : MonoBehaviour {

    List<TYPE_SUSTANCE> list_modules_added = new List<TYPE_SUSTANCE>();
    // Use this for initialization

    ModuleCounter module_counter = new ModuleCounter();

    [SerializeField]
    GUIMixer mixer_UI;

    [SerializeField]
    GWaveController actual_wave;
    public GWaveController ActualWave { get { return actual_wave; } }

    [SerializeField]
    GGameLogic game_logic;

    [SerializeField]
    AudioClip button_press;

    [SerializeField]
    AudioClip lever_press;

    void Awake()
    {
        GSingleton.Instance.SustanceMixer = this;
    }

    void Start() {

    }

    // Update is called once per frame
    void Update() {

    }

    public void PlayButtonSound()
    {
        AudioSource.PlayClipAtPoint(button_press, Camera.main.transform.position);
    }

    public void PlayLeverSound()
    {
        AudioSource.PlayClipAtPoint(button_press, Camera.main.transform.position);
    }

    public bool CanInject()
    {
        if (list_modules_added.Count > 0)
            return true;
        else
            return false;
    }

    public void InjectMix()
    {
        for (int i = 0; i < list_modules_added.Count; i++)
        {
            if (list_modules_added[i] == TYPE_SUSTANCE.ADD_AMPLITUDE)
            {
                actual_wave.AddAmplitude(1);
            }
            else if(list_modules_added[i] == TYPE_SUSTANCE.SUBSTRACT_AMPLITUDE)
            {
                actual_wave.AddAmplitude(-1);
            }
            else if (list_modules_added[i] == TYPE_SUSTANCE.ADD_LONGITUDE)
            {
                actual_wave.AddLongitude(1);
            }
            else if (list_modules_added[i] == TYPE_SUSTANCE.SUBSTRACT_LONGITUDE)
            {
                actual_wave.AddLongitude(-1);
            }
        }

        GSingleton.Instance.GameLogic.Inject();
        ResetMixer();
    }

    public void AddModule(TYPE_SUSTANCE _type_module)
    {
        list_modules_added.Add(_type_module);
        RefreshCounter();    
        RefreshUI();
    }

    public void ResetMixer()
    {
        list_modules_added.Clear();
        RefreshCounter();
        RefreshUI();
    }

    public void RefreshCounter()
    {
        module_counter.Reset();

        for (int i = 0; i < list_modules_added.Count; i++)
        {
            if (list_modules_added[i] == TYPE_SUSTANCE.ADD_LONGITUDE)
            {
                module_counter.add_longitude++;
            }
            else if(list_modules_added[i] == TYPE_SUSTANCE.SUBSTRACT_LONGITUDE)
            {
                module_counter.substract_longitude++;
            }
            else if (list_modules_added[i] == TYPE_SUSTANCE.ADD_AMPLITUDE)
            {
                module_counter.add_amplitude++;
            }
            else
            {
                module_counter.substract_amplitude++;
            }
        }

        module_counter.add_longitude = Mathf.Clamp(module_counter.add_longitude, 0, 5);
        module_counter.substract_longitude = Mathf.Clamp(module_counter.substract_longitude, 0, 5);
        module_counter.add_amplitude = Mathf.Clamp(module_counter.add_amplitude, 0, 5);
        module_counter.substract_amplitude = Mathf.Clamp(module_counter.substract_amplitude, 0, 5);
    }

    public void RefreshUI()
    {
        mixer_UI.RefreshModulesUI(module_counter);
    }
}

public class ModuleCounter
{
    public int add_longitude;
    public int substract_longitude;
    public int add_amplitude;
    public int substract_amplitude;

    public void Reset()
    {
        add_longitude = 0;
        substract_longitude = 0;
        add_amplitude = 0;
        substract_amplitude = 0;
    }

}
